import React, { useState, useRef, useEffect, useMemo } from 'react';
import { Settings } from './components/Settings';
import { RoundsHistory } from './components/RoundsHistory';
import { Instructions } from './components/Instructions';
import { saveState, loadState, exportToCSV } from './utils/storage';
import {
  Target,
  Stats,
  ViewportOffset,
  AccelerationSettings,
  Round,
  CONSTANTS,
  ZoomPreset,
  AppMode,
} from './types';
import { generateTargets, getRandomTarget } from './utils/target';
import { 
  calculateInitialViewport,
  clampZoom,
  calculateTouchDistance,
  calculateZoomFactor
} from './utils/interaction';
import { 
  createNewRound, 
  createInitialStats, 
  calculateAverageTime 
} from './utils/stats';
import { calculateAccelerationFactor, processWheelDelta, updateWheelSamples } from './utils/acceleration';
import { createDefaultSettings } from './utils/settings';
import { clamp } from './utils/math';
import './App.css';

function App() {
  // Simulation state
  const [targets, setTargets] = useState<Target[]>([]);
  const [currentTarget, setCurrentTarget] = useState<number | null>(null);
  const [zoomLevel, setZoomLevel] = useState<number>(CONSTANTS.ZOOM_LEVELS.default);
  const [stats, setStats] = useState<Stats>(createInitialStats());

  // Experiment mode configuration
  const [mode, setMode] = useState<AppMode>('demo');
  const [currentPreset, setCurrentPreset] = useState<ZoomPreset>('A');
  const [presetsCompleted, setPresetsCompleted] = useState<ZoomPreset[]>([]);
  const [targetCount, setTargetCount] = useState(0);
  const [studyComplete, setStudyComplete] = useState(false);

  const [settings, setSettings] = useState<AccelerationSettings>(() => {
    const savedState = loadState();
    const defaultSettings = createDefaultSettings();
    return savedState?.settings ?? defaultSettings;
  });

  const [rounds, setRounds] = useState<Round[]>(() => {
    const savedState = loadState();
    return savedState?.rounds ?? [];
  });

  // UI visibility states
  const [showSettings, setShowSettings] = useState(false);
  const [showRounds, setShowRounds] = useState(false);
  const [showInstructions, setShowInstructions] = useState(false);
  const [showResizeWarning, setShowResizeWarning] = useState(false);
  const [viewportOffset, setViewportOffset] = useState<ViewportOffset>({
    x: 0,
    y: 0,
  });

  // Refs for interaction tracking
  const isDragging = useRef(false);
  const dragStartTime = useRef(0);
  const dragStartPos = useRef({ x: 0, y: 0 });
  const DRAG_THRESHOLD = 5; // pixels

  const containerRef = useRef<HTMLDivElement>(null);
  const zoomSpeedSamples = useRef<number[]>([]);
  const touchStartDistance = useRef<number>(0);
  const wasReset = useRef(false);
  const resizeTimerRef = useRef<number | undefined>(undefined);

  // Add state for tracking preset transitions
  const [showPresetTransition, setShowPresetTransition] = useState(false);
  const [transitionPreset, setTransitionPreset] = useState<ZoomPreset | null>(null);

  // Load saved state on initial mount
  useEffect(() => {
    const savedState = loadState();
    if (savedState?.stats) {
      setStats(savedState.stats);
    }
    setShowInstructions(!savedState?.rounds || savedState.rounds.length === 0);
  }, []);

  // Persist state when it changes
  useEffect(() => {
    saveState({
      rounds,
      stats,
      settings,
    });
  }, [rounds, stats, settings]);

  // Initialize the simulation on first load
  useEffect(() => {
    initializeGame();
  }, []);

  const initializeGame = () => {
    const newTargets = generateTargets();
    setTargets(newTargets);
    setCurrentTarget(getRandomTarget(null, CONSTANTS.TARGET_COUNT));

    // Initialize at minimum zoom to show all targets
    const initialZoom = CONSTANTS.ZOOM_LEVELS.min;
    setZoomLevel(initialZoom);
    setViewportOffset(calculateInitialViewport(initialZoom));
    setStats(createInitialStats());
    
    setTargetCount(0);
    setPresetsCompleted([]);
    setStudyComplete(false);
  };

  // Configure settings when preset changes
  useEffect(() => {
    if (currentPreset) {
      setSettings(prev => ({
        ...prev,
        enabled: true,
        preset: currentPreset,
        accelerationCurve: [...CONSTANTS.PRESETS[currentPreset]]
      }));
    }
  }, [currentPreset]);

  // Modify the useEffect for preset progression to show the transition
  useEffect(() => {
    if (mode !== 'study') return;
    
    if (targetCount >= CONSTANTS.STUDY_TARGETS_PER_PRESET) {
      if (!presetsCompleted.includes(currentPreset)) {
        setPresetsCompleted(prev => [...prev, currentPreset]);
      }
      
      const allPresets: ZoomPreset[] = ['A', 'B', 'C', 'D'];
      const currentIndex = allPresets.indexOf(currentPreset);
      
      if (currentIndex < allPresets.length - 1) {
        // Show the transition to the next preset
        const nextPreset = allPresets[currentIndex + 1];
        setTransitionPreset(nextPreset);
        setShowPresetTransition(true);
        
        // Hide the transition after animation completes
        setTimeout(() => {
          setShowPresetTransition(false);
          setCurrentPreset(nextPreset);
          setTargetCount(0);
        }, 1500); // Match the animation duration
      } else {
        setStudyComplete(true);
      }
    }
  }, [targetCount, currentPreset, mode, presetsCompleted]);

  // Handle zoom wheel events and track timing data
  useEffect(() => {
    const container = containerRef.current;
    if (!container) return;

    const handleWheelEvent = (event: WheelEvent) => {
      if (event.ctrlKey || event.metaKey) {
        event.preventDefault();
    
        const containerBounds = container.getBoundingClientRect();
        const cursorX = event.clientX - containerBounds.left;
        const cursorY = event.clientY - containerBounds.top;

        const worldX = viewportOffset.x + cursorX / zoomLevel;
        const worldY = viewportOffset.y + cursorY / zoomLevel;

        const scrollAmount = processWheelDelta(-event.deltaY);
        const now = Date.now();
        
        const direction = scrollAmount > 0 ? 'in' : 'out';
        
        setStats(previous => {
          if (previous.isZooming && previous.zoomDirection === direction && previous.lastZoomTime) {
            const timeDelta = now - previous.lastZoomTime;
            // Only accumulate time if events are close together (continuous zooming)
            const shouldCount = timeDelta < 500;
            
            return {
              ...previous,
              lastZoomTime: now,
              zoomInTime: previous.zoomInTime + (direction === 'in' && shouldCount ? timeDelta : 0),
              zoomOutTime: previous.zoomOutTime + (direction === 'out' && shouldCount ? timeDelta : 0),
              zoomDirection: direction
            };
          }
          
          return {
            ...previous,
            isZooming: true,
            lastZoomTime: now,
            zoomDirection: direction
          };
        });
        
        if (isFinite(scrollAmount)) {
          zoomSpeedSamples.current = updateWheelSamples(zoomSpeedSamples.current, scrollAmount);
        }

        let zoomDelta = scrollAmount * 0.001;

        if (settings.enabled && zoomSpeedSamples.current.length > 0) {
          const accelerationFactor = calculateAccelerationFactor(
            zoomSpeedSamples.current,
            settings.accelerationCurve,
            settings.enabled
          );
          
          if (isFinite(accelerationFactor)) {
            zoomDelta *= accelerationFactor;
          }
        }

        zoomDelta = clamp(zoomDelta, -0.5, 0.5);
        const newZoom = clampZoom(zoomLevel * (1 + zoomDelta));

        if (isFinite(newZoom) && newZoom > 0) {
          const newOffsetX = worldX - cursorX / newZoom;
          const newOffsetY = worldY - cursorY / newZoom;

          if (isFinite(newOffsetX) && isFinite(newOffsetY)) {
            setZoomLevel(newZoom);
            setViewportOffset({ x: newOffsetX, y: newOffsetY });
          }
        }
      }
    };

    const handleZoomEnd = () => {
      setStats(previous => ({
        ...previous,
        isZooming: false,
        lastZoomTime: 0,
        zoomDirection: null
      }));
    };

    container.addEventListener('wheel', handleWheelEvent, { passive: false });
    
    // Detect when zooming stops after a brief pause
    let zoomEndTimer: number;
    container.addEventListener('wheel', () => {
      clearTimeout(zoomEndTimer);
      zoomEndTimer = window.setTimeout(handleZoomEnd, 300);
    });
    
    return () => {
      container.removeEventListener('wheel', handleWheelEvent);
      clearTimeout(zoomEndTimer);
    };
  }, [zoomLevel, viewportOffset, settings]);

  const handleTouchStart = (event: React.TouchEvent<HTMLDivElement>) => {
    if (event.touches.length === 2) {
      const touch1 = event.touches[0];
      const touch2 = event.touches[1];
      touchStartDistance.current = calculateTouchDistance(touch1, touch2);
    }
  };

  const handleTouchMove = (event: React.TouchEvent<HTMLDivElement>) => {
    if (event.touches.length === 2) {
      event.preventDefault();
      const touch1 = event.touches[0];
      const touch2 = event.touches[1];
      const currentDistance = calculateTouchDistance(touch1, touch2);
      const zoomFactor = calculateZoomFactor(currentDistance, touchStartDistance.current);
      setZoomLevel(previous => clampZoom(previous * zoomFactor));
    }
  };

  const handleTouchEnd = () => {
    touchStartDistance.current = 0;
  };

  const handleMouseMove = (event: React.MouseEvent) => {
    if (event.buttons === 1) {
      // Calculate total drag distance
      const dx = event.clientX - dragStartPos.current.x;
      const dy = event.clientY - dragStartPos.current.y;
      const dragDistance = Math.hypot(dx, dy);

      // Only set dragging if we've moved past the threshold
      if (dragDistance > DRAG_THRESHOLD) {
        isDragging.current = true;
      }

      setViewportOffset(previous => ({
        x: previous.x - event.movementX / zoomLevel,
        y: previous.y - event.movementY / zoomLevel,
      }));
    }
  };

  const handleMouseDown = (event: React.MouseEvent) => {
    dragStartTime.current = Date.now();
    dragStartPos.current = { x: event.clientX, y: event.clientY };
    isDragging.current = false;
  };

  const handleMouseUp = () => {
    // Reset drag state after a short delay to allow click event to check it
    setTimeout(() => {
      isDragging.current = false;
      dragStartPos.current = { x: 0, y: 0 };
    }, 0);
  };

  const handleClick = (event: React.MouseEvent) => {
    if (!containerRef.current || currentTarget === null || studyComplete) return;

    // If this was a drag operation or very slow click (likely a drag attempt), ignore it
    if (isDragging.current || Date.now() - dragStartTime.current > 200) {
      return;
    }

    const containerBounds = containerRef.current.getBoundingClientRect();
    const worldX = (event.clientX - containerBounds.left) / zoomLevel + viewportOffset.x;
    const worldY = (event.clientY - containerBounds.top) / zoomLevel + viewportOffset.y;

    const target = targets[currentTarget];
    if (!target) return;

    const distanceToTarget = Math.hypot(target.x - worldX, target.y - worldY);

    if (distanceToTarget <= target.radius) {
      const timeElapsed = Date.now() - stats.startTime;
      
      const newRound = createNewRound(
        timeElapsed, 
        stats.misclicks, 
        {
          ...settings,
          preset: mode === 'study' ? currentPreset : settings.preset
        },
        stats.zoomInTime,
        stats.zoomOutTime
      );
      
      setRounds(previous => [...previous, newRound]);
      
      setStats({
        startTime: Date.now(),
        misclicks: 0,
        targetsHit: stats.targetsHit + 1,
        times: [...stats.times, timeElapsed],
        zoomInTime: 0,
        zoomOutTime: 0,
        lastZoomTime: 0,
        isZooming: false,
        zoomDirection: null
      });
      
      setCurrentTarget(getRandomTarget(currentTarget, CONSTANTS.TARGET_COUNT));
      
      if (mode === 'study') {
        setTargetCount(prev => prev + 1);
      }
    } else {
      setStats(previous => ({
        ...previous,
        misclicks: previous.misclicks + 1,
      }));
    }
  };

  const handleReset = () => {
    if (window.confirm('Are you sure you want to reset all trials, history, and settings? This cannot be undone.')) {
      setRounds([]);
      setStats(createInitialStats());
      setSettings(createDefaultSettings());
      wasReset.current = true;
      initializeGame();
      
      setShowSettings(false);
      setShowRounds(false);
      setShowInstructions(true);
    }
  };

  const handleModeChange = (newMode: AppMode) => {
    if (newMode !== mode) {
      if (window.confirm('Changing modes will reset your current data. Continue?')) {
        setMode(newMode);
        setRounds([]);
        setStats(createInitialStats());
        
        if (newMode === 'study') {
          setTargetCount(0);
          setPresetsCompleted([]);
          setCurrentPreset('A');
          setStudyComplete(false);
        }
        
        initializeGame();
      }
    } else {
      setMode(newMode);
    }
  };

  const averageTime = calculateAverageTime(stats.times);

  // Regenerate targets when window is resized
  useEffect(() => {
    let lastWidth = window.innerWidth;
    let lastHeight = window.innerHeight;
    
    const handleResize = () => {
      if (mode !== 'study') {
        // Clear any existing timer
        if (resizeTimerRef.current) {
          window.clearTimeout(resizeTimerRef.current);
        }
        
        // Only regenerate if the size changed significantly
        const widthDiff = Math.abs(window.innerWidth - lastWidth);
        const heightDiff = Math.abs(window.innerHeight - lastHeight);
        const significantChange = widthDiff > 50 || heightDiff > 50;
        
        if (significantChange) {
          resizeTimerRef.current = window.setTimeout(() => {
            lastWidth = window.innerWidth;
            lastHeight = window.innerHeight;
            
            const newTargets = generateTargets();
            setTargets(newTargets);
            
            if (currentTarget !== null) {
              setCurrentTarget(getRandomTarget(currentTarget, CONSTANTS.TARGET_COUNT));
            }
            
            setShowResizeWarning(true);
          }, 800); // Longer debounce to prevent excessive regeneration
        }
      }
    };

    window.addEventListener('resize', handleResize);
    return () => {
      window.removeEventListener('resize', handleResize);
      if (resizeTimerRef.current) {
        window.clearTimeout(resizeTimerRef.current);
      }
    };
  }, [currentTarget, mode]);

  // Simplified target rendering that doesn't filter by viewport to ensure all targets are visible
  const renderedTargets = useMemo(() => {
    return targets.map((target, index) => (
      <div
        key={`target-${index}`}
        className={`target ${index === currentTarget ? 'active' : ''}`}
        style={{
          left: `${target.x}px`,
          top: `${target.y}px`,
          width: `${target.radius * 2}px`,
          height: `${target.radius * 2}px`,
          willChange: 'transform',
        }}
      />
    ));
  }, [targets, currentTarget]);

  return (
    <div className="app">
      <div className="stats">
        <span>
          <strong>Targets hit:</strong> {stats.targetsHit}
        </span>
        <span>
          <strong>Average time:</strong>{' '}
          {averageTime ? `${averageTime.toFixed(2)}ms` : '-'}
        </span>
        <span>
          <strong>Misclicks:</strong> {stats.misclicks}
        </span>
        {mode === 'study' && (
          <span>
            <strong>Preset:</strong> {currentPreset} ({targetCount}/{CONSTANTS.STUDY_TARGETS_PER_PRESET})
          </span>
        )}
      </div>

      {/* Add persistent preset indicator when in study mode */}
      {mode === 'study' && (
        <div className={`preset-indicator preset-${currentPreset}`}>
          <div>Preset</div>
          <div style={{ fontSize: '36px' }}>{currentPreset}</div>
          <div>{targetCount}/{CONSTANTS.STUDY_TARGETS_PER_PRESET}</div>
        </div>
      )}
      
      {/* Add preset transition overlay */}
      {showPresetTransition && transitionPreset && (
        <div className={`preset-transition-overlay preset-${transitionPreset}`}>
          <div className="preset-transition-content">
            Switching to Preset {transitionPreset}
          </div>
        </div>
      )}

      <div
        ref={containerRef}
        className="container"
        onMouseMove={handleMouseMove}
        onMouseDown={handleMouseDown}
        onMouseUp={handleMouseUp}
        onClick={handleClick}
        onTouchStart={handleTouchStart}
        onTouchMove={handleTouchMove}
        onTouchEnd={handleTouchEnd}
      >
        <div
          className="content"
          style={{
            transform: `translate(${-viewportOffset.x * zoomLevel}px, ${-viewportOffset.y * zoomLevel}px) scale(${zoomLevel})`,
            transformOrigin: '0 0',
          }}
        >
          {renderedTargets}
        </div>
      </div>

      <div className="controls">
        <button className="settings-toggle" onClick={() => setShowSettings(true)}>
          <span>⚙️</span> Settings
        </button>
        <button className="rounds-toggle" onClick={() => setShowRounds(true)}>
          <span>📊</span> History
        </button>
        <button
          className="export-button"
          onClick={() => exportToCSV(rounds)}
          disabled={rounds.length === 0}
        >
          <span>📤</span> Export Data
        </button>
        <button className="reset-button" onClick={handleReset}>
          <span>🔄</span> Reset
        </button>
      </div>

      <Settings
        settings={settings}
        onSettingsChange={setSettings}
        mode={mode}
        onModeChange={handleModeChange}
        visible={showSettings}
        onClose={() => setShowSettings(false)}
      />

      <RoundsHistory
        rounds={rounds}
        visible={showRounds}
        onClose={() => setShowRounds(false)}
      />

      <Instructions
        visible={showInstructions}
        onClose={() => setShowInstructions(false)}
        onModeSelect={handleModeChange}
      />

      {studyComplete && (
        <div className="break-screen">
          <div className="break-container">
            <h2>Study Complete!</h2>
            <p>You've completed all presets in the study (100 trials). Thank you for participating!</p>
            <p>You can now export your data or continue experimenting in demo mode.</p>
            <div className="button-group">
              <button onClick={() => {
                setStudyComplete(false);
                setMode('demo');
              }}>
                Switch to Demo Mode
              </button>
              <button
                onClick={() => exportToCSV(rounds)}
                disabled={rounds.length === 0}
                style={{ marginLeft: '10px' }}
              >
                Export Data
              </button>
            </div>
          </div>
        </div>
      )}

      {showResizeWarning && (
        <div className="resize-warning">
          <div className="resize-warning-content">
            <h3>Window Resized</h3>
            <p>Target positions have been reset to match your new window size.</p>
            <button onClick={() => setShowResizeWarning(false)}>Got it</button>
          </div>
        </div>
      )}
    </div>
  );
}

export default App;
