import React, { useState } from 'react';
import { AccelerationSettings, ZoomPreset, AppMode, CONSTANTS } from '../types';
import { updateSettingsCurve } from '../utils/settings';
import { clamp } from '../utils/math';
import './Settings.css';

interface SettingsProps {
	settings: AccelerationSettings;
	onSettingsChange: (settings: AccelerationSettings) => void;
	mode: AppMode;
	onModeChange: (mode: AppMode) => void;
	visible: boolean;
	onClose: () => void;
}

export const Settings: React.FC<SettingsProps> = ({
	settings,
	onSettingsChange,
	mode,
	onModeChange,
	visible,
	onClose,
}) => {
	const numSliders = 5;
	const [editingValue, setEditingValue] = useState<string[]>(
		new Array(numSliders).fill('')
	);

	const handleSliderChange = (index: number, value: number) => {
		const newSettings = updateSettingsCurve(settings, index, value);
		onSettingsChange(newSettings);
		setEditingValue(prev => {
			const next = [...prev];
			next[index] = '';
			return next;
		});
	};

	const handleInputChange = (index: number, value: string) => {
		setEditingValue(prev => {
			const next = [...prev];
			next[index] = value;
			return next;
		});
	};

	const handleInputBlur = (index: number) => {
		const value = parseFloat(editingValue[index]);
		if (!isNaN(value)) {
			const clampedValue = clamp(value, 1, 10);
			handleSliderChange(index, clampedValue);
		}
		setEditingValue(prev => {
			const next = [...prev];
			next[index] = '';
			return next;
		});
	};

	const handleInputKeyDown = (e: React.KeyboardEvent<HTMLInputElement>, index: number) => {
		if (e.key === 'Enter') {
			e.currentTarget.blur();
		} else if (e.key === 'Escape') {
			setEditingValue(prev => {
				const next = [...prev];
				next[index] = '';
				return next;
			});
			e.currentTarget.blur();
		}
	};

	const handlePresetChange = (preset: ZoomPreset) => {
		onSettingsChange({
			...settings,
			preset,
			accelerationCurve: [...CONSTANTS.PRESETS[preset]]
		});
	};

	const handleModeChange = (newMode: AppMode) => {
		onModeChange(newMode);
	};

	if (!visible) return null;

	return (
		<div className="settings-panel">
			<div className="settings-header">
				<h2>Settings</h2>
				<button onClick={onClose}>×</button>
			</div>

			<div className="mode-selection">
				<h3>Mode</h3>
				<div className="radio-group">
					<label>
						<input
							type="radio"
							name="mode"
							value="demo"
							checked={mode === 'demo'}
							onChange={() => handleModeChange('demo')}
						/>
						<span>Demo Mode</span>
					</label>
					<label>
						<input
							type="radio"
							name="mode"
							value="study"
							checked={mode === 'study'}
							onChange={() => handleModeChange('study')}
						/>
						<span>Study Mode (25 targets per preset)</span>
					</label>
				</div>
			</div>

			<div className="preset-selection">
				<h3>Preset Curves</h3>
				<div className="preset-buttons">
					{Object.keys(CONSTANTS.PRESETS).map((presetKey) => (
						<button
							key={presetKey}
							className={`preset-button ${settings.preset === presetKey ? 'active' : ''}`}
							onClick={() => handlePresetChange(presetKey as ZoomPreset)}
						>
							Preset {presetKey}
						</button>
					))}
				</div>
			</div>

			<div className="acceleration-toggle">
				<label>
					<input
						type="checkbox"
						checked={settings.enabled}
						onChange={(e) =>
							onSettingsChange({ ...settings, enabled: e.target.checked })
						}
					/>
					Enable Zoom Acceleration
				</label>
			</div>

			<div className="sliders-container">
				<h3>Acceleration Curve</h3>
				{Array.from({ length: numSliders }).map((_, i) => (
					<div key={i} className="slider-row">
						<label>{((i / (numSliders - 1)) * 2).toFixed(2)} zoom units/s</label>
						<input
							type="range"
							min="1"
							max="10"
							step="0.1"
							value={settings.accelerationCurve?.[i] ?? 1}
							onChange={(e) => handleSliderChange(i, parseFloat(e.target.value))}
							disabled={!settings.enabled}
						/>
						<input
							type="text"
							className="value-input"
							value={editingValue[i] || (settings.accelerationCurve?.[i] ?? 1).toFixed(1)}
							onChange={(e) => handleInputChange(i, e.target.value)}
							onBlur={() => handleInputBlur(i)}
							onKeyDown={(e) => handleInputKeyDown(e, i)}
							disabled={!settings.enabled}
							placeholder={(settings.accelerationCurve?.[i] ?? 1).toFixed(1)}
						/>
					</div>
				))}
			</div>
		</div>
	);
};
