import { ViewportOffset, CONSTANTS } from '../types';
import { clamp, calculateDistance } from './math';

// Zoom-related functions
export const calculateZoomDelta = (
  wheelDelta: number,
  accelerationFactor: number,
  baseMultiplier = 0.001
): number => clamp(wheelDelta * baseMultiplier * accelerationFactor, -0.5, 0.5);

export const calculateInitialViewport = (initialZoom: number): ViewportOffset => {
  // When fully zoomed out, we want to see the entire content area
  // Account for safety margins and UI exclusion zones
  const windowWidth = window.innerWidth;
  const windowHeight = window.innerHeight;
  
  // Same UI margins as in target generation
  const UI_TOP_MARGIN = 100;
  const UI_BOTTOM_MARGIN = 50; // Added bottom margin
  
  // We need to account for the safety margin used in target generation
  const MAX_RADIUS = CONSTANTS.TARGET_MAX_RADIUS;
  const safetyMargin = MAX_RADIUS * 1.5; // Increased from 1.2 to 1.5
  
  // Calculate the full target area including margins
  const minZoom = CONSTANTS.ZOOM_LEVELS.min;
  const areaWidth = windowWidth / minZoom;
  const areaHeight = (windowHeight - UI_TOP_MARGIN - UI_BOTTOM_MARGIN) / minZoom;
  
  // Account for the UI exclusion zone in Y offset
  const startY = UI_TOP_MARGIN / minZoom;
  
  // Center viewport on the usable area
  const usableWidth = areaWidth - (safetyMargin * 2);
  const usableHeight = areaHeight - (safetyMargin * 2);
  const usableCenterX = safetyMargin + (usableWidth / 2);
  const usableCenterY = startY + safetyMargin + (usableHeight / 2);
  
  // Position viewport so that the center of the usable area is in the middle of the screen
  return {
    x: usableCenterX - (windowWidth / initialZoom / 2),
    y: usableCenterY - (windowHeight / initialZoom / 2)
  };
};

export const clampZoom = (zoom: number): number => 
  clamp(zoom, CONSTANTS.ZOOM_LEVELS.min, CONSTANTS.ZOOM_LEVELS.max);

// Touch-related functions
export const calculateTouchDistance = (touch1: React.Touch | Touch, touch2: React.Touch | Touch): number =>
  calculateDistance(touch1.clientX, touch1.clientY, touch2.clientX, touch2.clientY);

export const calculateZoomFactor = (currentDistance: number, startDistance: number): number =>
  startDistance > 0 ? currentDistance / startDistance : 1; 