import { Target, CONSTANTS } from '../types';

export const generateTargets = () => {
	const newTargets: Target[] = [];
	
	// Use the constants from types.ts for consistency
	const MIN_RADIUS = CONSTANTS.TARGET_MIN_RADIUS;
	const MAX_RADIUS = CONSTANTS.TARGET_MAX_RADIUS;
	
	// Get browser window dimensions
	const windowWidth = window.innerWidth;
	const windowHeight = window.innerHeight;
	
	// Define UI exclusion zones
	const UI_TOP_MARGIN = 100; // pixels from top to exclude
	const UI_BOTTOM_MARGIN = 50; // pixels from bottom to exclude for any potential UI
	
	// Base the target area on the minimum zoom level
	// This ensures when fully zoomed out, we see the entire grid
	const minZoom = CONSTANTS.ZOOM_LEVELS.min;
	const areaWidth = windowWidth / minZoom;
	const areaHeight = (windowHeight - UI_TOP_MARGIN - UI_BOTTOM_MARGIN) / minZoom;
	
	// Start Y position below the UI elements
	const startY = UI_TOP_MARGIN / minZoom;
	
	// Create a grid-like distribution
	const gridSize = Math.ceil(Math.sqrt(CONSTANTS.TARGET_COUNT));
	
	// Increase safety margin to prevent any clipping
	const safetyMargin = MAX_RADIUS * 1.5; // Increased from 1.2 to 1.5
	
	// Adjust the usable area to prevent clipping at edges
	const usableWidth = areaWidth - (safetyMargin * 2);
	const usableHeight = areaHeight - (safetyMargin * 2);
	
	// New starting point that accounts for target radius
	const startUsableX = safetyMargin;
	const startUsableY = startY + safetyMargin;
	
	// Calculate adjusted cell dimensions for the usable area
	const adjustedCellWidth = usableWidth / gridSize;
	const adjustedCellHeight = usableHeight / gridSize;
	
	for (let i = 0; i < CONSTANTS.TARGET_COUNT; i++) {
		// Calculate grid position
		const gridX = i % gridSize;
		const gridY = Math.floor(i / gridSize);
		
		// Reduce randomness to keep targets more centered in their cells
		const randomOffsetX = (Math.random() - 0.5) * adjustedCellWidth * 0.4; // Reduced from 0.6
		const randomOffsetY = (Math.random() - 0.5) * adjustedCellHeight * 0.4; // Reduced from 0.6
		
		// Calculate final position - add margin to prevent edge clipping
		const x = startUsableX + adjustedCellWidth * (gridX + 0.5) + randomOffsetX;
		const y = startUsableY + adjustedCellHeight * (gridY + 0.5) + randomOffsetY;
		
		// Random radius
		const radius = MIN_RADIUS + Math.random() * (MAX_RADIUS - MIN_RADIUS);
		
		// Add the target to the array
		newTargets.push({
			x,
			y,
			radius
		});
	}
	
	return newTargets;
};

export const getRandomTarget = (currentTarget: number | null, maxTargets: number): number => {
	let newTarget: number;
	do {
		newTarget = Math.floor(Math.random() * maxTargets);
	} while (newTarget === currentTarget);
	return newTarget;
};
